@@ -284,7 +284,9 @@ void cmd_info(char *cmdbuf) {
 	cprintf("0\n"); /* 1 = no, this server is not LDAP-enabled */
 #endif
 
-	if (config.c_auth_mode == AUTHMODE_NATIVE) {
+	if ((config.c_auth_mode == AUTHMODE_NATIVE) &&
+	    (config.c_disable_newu == 0))
+	{
 		cprintf("%d\n", config.c_disable_newu);
 	}
 	else {
