@@ -153,9 +153,10 @@ eNextState FinalizeMessageSend_DB(AsyncIO *IO)
 
 	Msg->IDestructQueItem = DecreaseQReference(Msg->MyQItem);
 
-	Msg->nRemain = CountActiveQueueEntries(Msg->MyQItem);
+	Msg->nRemain = CountActiveQueueEntries(Msg->MyQItem, 0);
 
 	if (Msg->MyQEntry->Active && 
+	    !Msg->MyQEntry->StillActive &&
 	    CheckQEntryIsBounce(Msg->MyQEntry))
 	{
 		/* are we casue for a bounce mail? */
@@ -176,7 +177,7 @@ eNextState FinalizeMessageSend_DB(AsyncIO *IO)
 	Msg->MyQItem->QueMsgID = -1;
 
 	if (Msg->IDestructQueItem)
-		smtpq_do_bounce(Msg->MyQItem, Msg->msgtext);
+		smtpq_do_bounce(Msg->MyQItem, Msg->msgtext, Msg->pCurrRelay);
 
 	if (Msg->nRemain > 0)
 	{
