@@ -246,7 +246,7 @@ int CheckQEntryIsBounce(MailQEntry *ThisItem)
 		return 0;
 }	
 
-int CountActiveQueueEntries(OneQueItem *MyQItem)
+int CountActiveQueueEntries(OneQueItem *MyQItem, int before)
 {
 	HashPos  *It;
 	long len;
@@ -258,15 +258,20 @@ int CountActiveQueueEntries(OneQueItem *MyQItem)
 	It = GetNewHashPos(MyQItem->MailQEntries, 0);
 	while (GetNextHashPos(MyQItem->MailQEntries, It, &len, &Key, &vQE))
 	{
+		int Active;
 		MailQEntry *ThisItem = vQE;
 
 		if (CheckQEntryActive(ThisItem))
 		{
 			ActiveDeliveries++;
-			ThisItem->Active = 1;
+			Active = 1;
 		}
 		else
-			ThisItem->Active = 0;
+			Active = 0;
+		if (before)
+			ThisItem->Active = Active;
+		else
+			ThisItem->StillActive = Active;
 	}
 	DeleteHashPos(&It);
 	return ActiveDeliveries;
@@ -505,7 +510,7 @@ StrBuf *smtp_load_msg(OneQueItem *MyQItem, int n)
  * instructions for "5" codes (permanent fatal errors) and produce/deliver
  * a "bounce" message (delivery status notification).
  */
-void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt)
+void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt, ParsedURL *Relay)
 {
 	static int seq = 0;
 
@@ -556,6 +561,7 @@ void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt)
 		{
 			++num_bounces;
 
+			StrBufAppendBufPlain(Msg, HKEY(" "), 0);
 			StrBufAppendBuf(Msg, ThisItem->Recipient, 0);
 			StrBufAppendBufPlain(Msg, HKEY(": "), 0);
 			StrBufAppendBuf(Msg, ThisItem->StatusMessage, 0);
@@ -572,6 +578,18 @@ void smtpq_do_bounce(OneQueItem *MyQItem, StrBuf *OMsgTxt)
 		return;
 	}
 
+	if ((StrLength(MyQItem->SenderRoom) == 0) && MyQItem->HaveRelay) {
+		/* one message that relaying is broken is enough; no extra room error message. */
+		StrBuf *RelayDetails = NewStrBuf();
+
+		StrBufPrintf(RelayDetails,
+			     "Relaying via %s failed permanently. \n Reason:\n%s\n Revalidate your relay configuration.",
+			     ChrPtr(Relay->URL),
+			     ChrPtr(Msg));
+                CtdlAideMessage(ChrPtr(RelayDetails), "Relaying Failed");
+		FreeStrBuf(&RelayDetails);
+	}
+
 	boundary = NewStrBufPlain(HKEY("=_Citadel_Multipart_"));
 	StrBufAppendPrintf(boundary,
 			   "%s_%04x%04x",
@@ -884,7 +902,7 @@ void smtp_do_procmsg(long msgnum, void *userdata) {
 	DeleteHashPos(&It);
 
 	MyQItem->NotYetShutdownDeliveries = 
-	MyQItem->ActiveDeliveries = CountActiveQueueEntries(MyQItem);
+		MyQItem->ActiveDeliveries = CountActiveQueueEntries(MyQItem, 1);
 
 	/* failsafe against overload: 
 	 * will we exceed the limit set? 
