@@ -2,21 +2,15 @@
  * This file contains functions which implement parts of the
  * text-mode user interface.
  *
- * Copyright (c) 1987-2010 by the citadel.org team
+ * Copyright (c) 1987-2012 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -67,6 +61,7 @@
 #include "rooms.h"
 #include "client_chat.h"
 #include "citadel_dirs.h"
+#include "tuiconfig.h"
 #ifndef HAVE_SNPRINTF
 #include "snprintf.h"
 #endif
@@ -97,7 +92,6 @@ char rc_url_cmd[SIZ];
 char rc_open_cmd[SIZ];
 char rc_gotmail_cmd[SIZ];
 
-char *gl_string;
 int next_lazy_cmd = 5;
 
 extern int screenwidth, screenheight;
@@ -492,91 +486,86 @@ int yesno_d(int d)
 
 
 
-/* Gets a line from the terminal */
-/* string == Pointer to string buffer */
-/* lim == Maximum length - if negative, no-show */
-void ctdl_getline(char *string, int lim) 
+/*
+ * Function to read a line of text from the terminal.
+ *
+ * string		Pointer to string buffer
+ * lim			Maximum length
+ * noshow		Echo asterisks instead of keystrokes?
+ * bs			Allow backspacing out of the prompt? (returns -1 if this happens)
+ *
+ * returns: string length
+ */
+int ctdl_getline(char *string, int lim, int noshow, int bs)
 {
-	int a, b;
-	char flag = 0;
+	int pos = strlen(string);
+	int ch;
 
-	if (lim < 0) {
-		lim = (0 - lim);
-		flag = 1;
-	}
-	strcpy(string, "");
-	gl_string = string;
 	async_ka_start();
+	if (noshow && !IsEmptyStr(string)) {
+		int num_stars = strlen(string);
+		while (num_stars--) {
+			scr_putc('*');
+		}
+	}
+	else {
+		scr_printf("%s", string);
+	}
 
-GLA:	a = inkey();
-	/* a = (a & 127); ** commented out because it isn't just an ASCII world anymore */
-	if ((a == 8 || a == 23) && (IsEmptyStr(string)))
-		goto GLA;
-	if ((a != 10) && (a != 8) && (strlen(string) == lim))
-		goto GLA;
-	if ((a == 8) && (string[0] != 0)) {
-		string[strlen(string) - 1] = 0;
-		scr_putc(8); scr_putc(32); scr_putc(8);
-		goto GLA;
-	}
-	if ((a == 23) && (string[0] != 0)) {
-		do {
-			string[strlen(string) - 1] = 0;
+	while(1) {
+		ch = inkey();
+
+		if ((ch == 8)  && (pos > 0)) {				/* backspace */
+			--pos;
 			scr_putc(8); scr_putc(32); scr_putc(8);
-		} while (!IsEmptyStr(string) && string[strlen(string) - 1] != ' ');
-		goto GLA;
+		}
+
+		else if ((ch == 8) && (pos == 0) && (bs)) {		/* backspace out of the prompt */
+			async_ka_end();
+			return(-1);
+		}
+
+		else if ((ch == 23) && (pos > 0)) {			/* Ctrl-W deletes a word */
+			while ((pos > 0) && !isspace(string[pos])) {
+				--pos;
+				scr_putc(8); scr_putc(32); scr_putc(8);
+			}
+			while ((pos > 0) && !isspace(string[pos-1])) {
+				--pos;
+				scr_putc(8); scr_putc(32); scr_putc(8);
+			}
+		}
+
+		else if (ch == 10) {					/* return */
+			string[pos] = 0;
+			scr_printf("\n");
+			async_ka_end();
+			return(pos);
+		}
+
+		else if (isprint(ch)) {					/* payload characters */
+			scr_putc((noshow ? '*' : ch));
+			string[pos] = ch;
+			++pos;
+		}
 	}
-	if ((a == 10)) {
-		scr_putc(10);
-		async_ka_end();
-		return;
-	}
-	if (a < 32)
-		a = '.';
-	b = strlen(string);
-	string[b] = a;
-	string[b + 1] = 0;
-	if (flag == 0)
-		scr_putc(a);
-	if (flag == 1)
-		scr_putc('*');
-	goto GLA;
 }
 
 
-/*
- * strprompt()  -  prompt for a string, print the existing value and
- *                 allow the user to press return to keep it...
+/* 
+ * newprompt()		prompt for a string, print the existing value, and
+ *			allow the user to press return to keep it...
+ *			If len is negative, pass the "noshow" flag to ctdl_getline()
  */
 void strprompt(char *prompt, char *str, int len)
 {
-	int i;
-	char buf[128];
-
 	print_instant();
 	color(DIM_WHITE);
-	scr_printf("%s ", prompt);
-	color(DIM_MAGENTA);
-	scr_printf("[");
-	color(BRIGHT_MAGENTA);
-
-	if (len >= 0) {
-		scr_printf("%s", str);
-	}
-	else {
-		for (i=0; !IsEmptyStr(&str[i]); ++i) {
-			scr_printf("*");
-		}
-	}
-
-	color(DIM_MAGENTA);
-	scr_printf("]");
+	scr_printf("%s", prompt);
 	color(DIM_WHITE);
 	scr_printf(": ");
 	color(BRIGHT_CYAN);
-	ctdl_getline(buf, len);
-	if (buf[0] != 0)
-		strcpy(str, buf);
+	ctdl_getline(str, abs(len), (len<0), 0);
 	color(DIM_WHITE);
 }
 
@@ -631,15 +620,17 @@ int intprompt(char *prompt, int ival, int imin, int imax)
 }
 
 /* 
- * newprompt()  -  prompt for a string with no existing value
- *                 (clears out string buffer first)
+ * newprompt()		prompt for a string with no existing value
+ *			(clears out string buffer first)
+ *			If len is negative, pass the "noshow" flag to ctdl_getline()
  */
 void newprompt(char *prompt, char *str, int len)
 {
+	str[0] = 0;
 	color(BRIGHT_MAGENTA);
 	scr_printf("%s", prompt);
 	color(DIM_MAGENTA);
-	ctdl_getline(str, len);
+	ctdl_getline(str, abs(len), (len<0), 0);
 	color(DIM_WHITE);
 }
 
@@ -660,18 +651,14 @@ void load_command_set(void)
 {
 	FILE *ccfile;
 	char buf[1024];
-	char editor_key[100];
 	struct citcmd *cptr;
 	struct citcmd *lastcmd = NULL;
 	int a, d;
 	int b = 0;
-	int i;
-
 
 	/* first, set up some defaults for non-required variables */
 
-	for (i = 0; i < MAX_EDITORS; i++)
-		strcpy(editor_paths[i], "");
+	strcpy(editor_path, "");
 	strcpy(printcmd, "");
 	strcpy(imagecmd, "");
 	strcpy(rc_username, "");
@@ -691,7 +678,6 @@ void load_command_set(void)
 #ifdef HAVE_OPENSSL
 	rc_encrypt = RC_DEFAULT;
 #endif
-	rc_alt_semantics = 0;
 
 	/* now try to open the citadel.rc file */
 
@@ -736,14 +722,8 @@ void load_command_set(void)
 #endif
 		}
 
-		if (!strncasecmp(buf, "editor=", 7))
-			strcpy(editor_paths[0], &buf[7]);
-
-		for (i = 0; i < MAX_EDITORS; i++)
-		{
-			sprintf(editor_key, "editor%d=", i);
-			if (!strncasecmp(buf, editor_key, strlen(editor_key)))
-				strcpy(editor_paths[i], &buf[strlen(editor_key)]);
+		if (!strncasecmp(buf, "editor=", 7)) {
+			strcpy(editor_path, &buf[7]);
 		}
 
 		if (!strncasecmp(buf, "printcmd=", 9))
@@ -789,6 +769,10 @@ void load_command_set(void)
 			if (!strncasecmp(&buf[11], "user", 4))
 				rc_ansi_color = 3;	/* user config */
 		}
+		if (!strncasecmp(buf, "status_line=", 12)) {
+			if (!strncasecmp(&buf[12], "on", 2))
+				enable_status_line = 1;
+		}
 		if (!strncasecmp(buf, "use_background=", 15)) {
 			if (!strncasecmp(&buf[15], "on", 2))
 				rc_color_use_bg = 9;
@@ -814,15 +798,6 @@ void load_command_set(void)
 		if (!strncasecmp(buf, "gotmailcmd=", 11))
 			strcpy(rc_gotmail_cmd, &buf[11]);
 
-		if (!strncasecmp(buf, "alternate_semantics=", 20)) {
-			if (!strncasecmp(&buf[20], "yes", 3)) {
-				rc_alt_semantics = 1;
-			}
-			else {
-				rc_alt_semantics = 0;
-			}
-		}
-
 		if (!strncasecmp(buf, "cmd=", 4)) {
 			strcpy(buf, &buf[4]);
 
@@ -1070,7 +1045,8 @@ int getcmd(CtdlIPC *ipc, char *argbuf)
 			if (cmdmatch(cmdbuf, cptr, 5)) {
 				/* We've found our command. */
 				if (requires_string(cptr, cmdpos)) {
-					ctdl_getline(argbuf, 64);
+					argbuf[0] = 0;
+					ctdl_getline(argbuf, 64, 0, 0);
 				} else {
 					scr_printf("\n");
 				}
